package aLibrary;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.Serializable;

/*	ARoundRectangle objects are designed for displaying rectangles
    Author:  David Riley -- July, 2000 */
    
/*	class invariant
    	getWidth() >= 0  AND  getHeight() >= 0  */
public class ARoundRectangle extends AView implements Serializable  {
	// The variables implement a unique ID number for each ARoundRectangle 
	private static int idShared = 0;
	private int idNum;


// Constructor methods
	/*	postcondition 
   			an ARectangle object is instantiated.
   			AND  isFilled is false
			AND  getColor() == Color.black 
			AND  getX() == 0   AND  getY() == 0   
			AND  getWidth() == 10  AND  getHeight() == 10   */
	public ARoundRectangle()  {
		super();
		ARoundRectangle.idShared++;
		idNum = ARoundRectangle.idShared;
	}  

	/*	postcondition 
   			an ARectangle object is instantiated.
   			AND  isFilled is false
			AND  getColor() == Color.black 
			AND  getX() == x   AND  getY() == y   
			AND  getWidth() == w  AND  getHeight() == h   */
	public ARoundRectangle(int x, int y, int w, int h)  {
		super(x, y, w, h);
		ARoundRectangle.idShared++;
		idNum = ARoundRectangle.idShared;
	}  
   
// Class Methods 
	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
		String  result = "ARoundRectangle object .... (#RR" + idNum + ")" + "\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() + "\n";
		if (parentContainer() == null)   {
   			result = result + "     not placed, ";
   		} else   {
   			result = result + "     is placed, ";
   		}
		if (isFilled())   {
   			result = result + "is filled, ";
   		} else   {
   			result = result + "is outlined, ";
   		}
   		result = result + " color: "+colorString(getColor());
		return result;
   	}	
   
// private methods
	private String colorString( Color c )   {
		if (c == Color.black)   {
			return "black";
		} else if (c == Color.blue)   {
			return "blue";
		} else if (c == Color.cyan)   {
			return "cyan";
		} else if (c == Color.darkGray)   {
			return "dark gray";
		} else if (c == Color.gray)   {
			return "gray";
		} else if (c == Color.green)   {
			return "green";
		} else if (c == Color.lightGray)   {
			return "light gray";
		} else if (c == Color.magenta)   {
			return "magenta";
		} else if (c == Color.orange)   {
			return "orange";
		} else if (c == Color.pink)   {
			return "pink";
		} else if (c == Color.red)   {
			return "red";
		} else if (c == Color.white)   {
			return "white";
		} else if (c == Color.yellow)   {
			return "yellow";
		} else {
			return "redness: "+c.getRed() + "   greeness: "+c.getGreen()+"   blueness: "+c.getBlue();
		}
	}

	private int min(int a, int b)  {
		if (a < b)
			return a;
		else
			return b;
	}  


// Auxiliary Methods

	/*	postcondition
   			this rectangle is drawn as a filled shape with dimensions
			given by getX(), getY(), getWidth(), getHeight(),
			and color given by getColor() */
	public void fill(Graphics g)  {
		g.fillRoundRect(0, 0, getWidth()-1, getHeight()-1,
                 min(getWidth(), getHeight())/2, 
                 min(getWidth(), getHeight())/2);
                // The "-1"s should be removed from the line above if setClip is corrected
	} 

	/*	postcondition
   			this rectangle is drawn as an outline with dimensions
			given by getX(), getY(), getWidth(), getHeight(),
			and color given by getColor() */
	public void stroke(Graphics g)  {
		g.drawRoundRect(0, 0, getWidth()-1, getHeight()-1,
                 min(getWidth(), getHeight())/2, 
                 min(getWidth(), getHeight())/2);
                // The "-1"s should be removed from the line above if setClip is corrected
	} 

}
