package aLibrary;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.lang.*;
import java.io.Serializable;


/*	AView objects are designed to both display and input 
	single lines of text.
    Author:  David Riley -- July, 2000 */
    
/*	class invariant
    	getWidth() >= 0  AND  getHeight() >= 0  */
public class ATextField extends JTextField  
	 implements AShape,  Serializable, ActionListener  {

// Attributes   
	public boolean translationErrorOccurred;
   	
	// The variables below implement a unique ID number for each ARectangle 
	private static int idShared = 0;
	private int idNum;
   
	/*	postcondition
   			result == the string value contained within the field
   			AND  !translationErrorOccurred */
	public String getText()  {
		translationErrorOccurred = false;
		return super.getText();
	}  
   
	/*	postcondition
   			getText() is a valid constant of type int
   				IMPLIES
   					result == the int value contained within the field
   					AND  !translationErrorOccurred
    		getText() is NOT a valid constant of type int
   				IMPLIES
   					result == 0
   					AND  translationErrorOccurred  */
	public int getInt()  {
		try{
			translationErrorOccurred = false;
			return (Integer.valueOf(super.getText())).intValue();
		}
		catch (NumberFormatException e)  {
			translationErrorOccurred = true;
			return 0;
		}
	} 
   
	/*	postcondition
   			getText() is a valid constant of type float
   				IMPLIES
   					result == the int value contained within the field
   					AND  !translationErrorOccurred
    		getText() is NOT a valid constant of type float
   				IMPLIES
   					result == 0.0
   					AND  translationErrorOccurred  */
	public float getFloat()  {
		try{
			translationErrorOccurred = false;
			return (Float.valueOf(super.getText())).floatValue();
		}
		catch (NumberFormatException e)  {
			translationErrorOccurred = true;
			return 0;
		}
	}
   
	/*	postcondition
   			getText() is a valid constant of type double
   				IMPLIES
   					result == the int value contained within the field
   					AND  !translationErrorOccurred
    		getText() is NOT a valid constant of type float
   				IMPLIES
   					result == 0.0
   					AND  translationErrorOccurred  */
	public double getDouble()  {
		try{
			translationErrorOccurred = false;
			return (Double.valueOf(super.getText())).doubleValue();
		}
		catch (NumberFormatException e)  {
			translationErrorOccurred = true;
			return 0;
		}
	} 
   
	/*	postcondition
   			getText() is "true" or "false"
   				IMPLIES
   					result == the int value contained within the field
   					AND  !translationErrorOccurred
    		getText() is NOT "true" or "false"
   				IMPLIES
   					result == false
   					AND  translationErrorOccurred  */
	public boolean getBoolean()  {
		if ("FALSE".equalsIgnoreCase(super.getText().trim()) )   {
			translationErrorOccurred = false;
			return false;
		} else if ("TRUE".equalsIgnoreCase(super.getText().trim()) )   {
			translationErrorOccurred = false;
			return true;
		} else {
			translationErrorOccurred = true;
			return false;
		}
	} 
   
	/* 	postcondition
   			result == the parent object upon which this is placed   */
	public Container parentContainer()  {
		Container c;
   		c = getParent();
   		if (c instanceof APanel)   {
   			return ((APanel)c).getAWindow();
   		} else {
   			return c;
   		}
	} 


// Constructor methods
	/*	postcondition 
   			this text field is instantiated
			AND  getX() == 0   AND  getY() == 0   
			AND  getWidth() == 100  AND  getHeight() == 18   
			AND  getForeground() == Color.black 
			AND  getBackground() == Color.lightGray
			AND  !translationErrorOccurred */
	public ATextField()  {
		super("");
		setBounds(0, 0, 100, 18);
		setForeground(Color.black);
		setBackground(Color.lightGray);
		Font newFont = new Font( "Courier", Font.PLAIN, 10 );
		setFont( newFont );
		translationErrorOccurred = false;
		addActionListener(this);
		idShared++;
		idNum = idShared;
	} 
	
	/*	postcondition 
   			this text field is instantiated
			AND  getX() == x   AND  getY() == y   
			AND  getWidth() == w  AND  getHeight() == h   
			AND  getForeground() == Color.black 
			AND  getBackground() == Color.lightGray
			AND  !translationErrorOccurred */
	public ATextField(int x, int y, int w, int h)  {
		super("");
		setBounds(x, y, w, h);
		setForeground(Color.black);
		setBackground(Color.lightGray);
		Font newFont = new Font( "Courier", Font.PLAIN, 10 );
		setFont( newFont );
		translationErrorOccurred = false;
		addActionListener(this);
		idShared++;
		idNum = idShared;
	} 


// Class Methods
	/* 	postcondition
   			this is added to the c container */
	public void place(Container c)  {
		c.add(this,0);  //zero places at front of list
	}
	
	/* 	postcondition
   			this is removed from where it is placed  
      		AND  parentShape() == null  */
	public void remove()  {
		if  (getParent() != null)
			getParent().remove(this);
	} 

	/* 	precondition
   			sw >= 0.0  AND  sh >= 0.0   
     	 postcondition
      		getWidth() == old getWidth() * sw   
      		AND   getHeight() == old getHeight() * sh */
	public void scale(double sw, double sh)  {
		setBounds(getX(), getY(),
            	(int)(getWidth() * sw),
             	(int)(getHeight() * sh));
	} 

	/* 	postcondition
   			getX() == old getX() + tx   
   			AND   getY() == old getY() + ty  */
	public void translate(double tx, double ty)  {
		setLocation((int)(getX() + tx), (int)(getY() + ty));
	} 
  
	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
		String  result = "ATextField object .... (#TF" + idNum + ")" + "\n";
   		result = result + "     text: " + "\""+getText() +"\"" + "\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() + "\n";
		if (parentContainer() == null)   {
   			result = result + "     not placed, ";
   		} else   {
   			result = result + "     is placed, ";
   		}
   		result = result + "     foreground color: "+colorString(getForeground());
   		result = result + ",   background color: "+colorString(getBackground());
		return result;
   	}	
   	
    /*	postcondition
   			The size of the characters is set to a font size of s */
	public void setFontSize( int s )  {
		Font newFont = new Font( getFont().getName(), Font.PLAIN, s );
		setFont( newFont );
	} 
   
    /*	postcondition
   			result == font size for this */
	public int getFontSize( int s )  {
		return getFont().getSize();
	} 



// Action event handler
   public void actionPerformed(ActionEvent e)
   {}  //actionPerformed

   	
// private methods
	private String colorString( Color c )   {
		if (c == Color.black)   {
			return "black";
		} else if (c == Color.blue)   {
			return "blue";
		} else if (c == Color.cyan)   {
			return "cyan";
		} else if (c == Color.darkGray)   {
			return "dark gray";
		} else if (c == Color.gray)   {
			return "gray";
		} else if (c == Color.green)   {
			return "green";
		} else if (c == Color.lightGray)   {
			return "light gray";
		} else if (c == Color.magenta)   {
			return "magenta";
		} else if (c == Color.orange)   {
			return "orange";
		} else if (c == Color.pink)   {
			return "pink";
		} else if (c == Color.red)   {
			return "red";
		} else if (c == Color.white)   {
			return "white";
		} else if (c == Color.yellow)   {
			return "yellow";
		} else {
			return "redness: "+c.getRed() + "   greeness: "+c.getGreen()+"   blueness: "+c.getBlue();
		}
	}
}