// DateAndTime class
//    This class is grants access to the current clock date and time.
//
//    Author: David Riley  --  September, 1999
package aLibrary;

import java.util.*;
import java.io.Serializable;

public class DateAndTime implements  Serializable  {
	// U.S. Central Standard Time is GMT - 6 hours.
	// (The following constant should be edited for different time zones.)
	protected static final int localTimeOffset = -6;
	
    public DateAndTime()  {};
    
    private static Calendar currentDateTime()  {
   		// get the supported ids for GMT+timeOffset   		
        String[] ids = TimeZone.getAvailableIDs(localTimeOffset * 60 * 60 * 1000);
        if (ids.length == 0)   {
            System.out.println("ERROR - unable to run DateAndTime for the specified time zone.");
            System.out.println("        (Possible fix: changing localTimeOffset constant.)");
            System.exit(0);
        }
        // create a local time zone
        SimpleTimeZone localTimeZone = new SimpleTimeZone(localTimeOffset * 60 * 60 * 1000, ids[0]);
        // set up rules for daylight savings time
		// (The following methods may need to be edited for different time zones.)
        localTimeZone.setStartRule(Calendar.APRIL, 1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
        localTimeZone.setEndRule(Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
        return new GregorianCalendar(localTimeZone);
    }
    
	/*	postcondition 
			result == current calendar year */
    public static int year() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.YEAR);
    }
    
    /*	postcondition 
    		result == current month (Jan returns 1, etc.) */
    public static int month() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.MONTH);
    }
    
    /*	postcondition 
    		result == current day of the month */
    public static int day() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.DAY_OF_MONTH);
    }
    
    /*	postcondition 
    		result == current hour (in military time) */
    public static int hour() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.HOUR_OF_DAY);
    }
    
    /*	postcondition 
    		result == current minutes past hour() */
    public static int minute() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.MINUTE);
    }
    
    /*	postcondition 
    		result == current seconds past minute() */
    public static int second() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.SECOND);
    }
    
    /*	postcondition 
    		result == current milliseconds past second() */
    public static int millisecond() {
    	Calendar rightNow = currentDateTime();
    	return rightNow.get(Calendar.MILLISECOND);
    }
}