package aLibrary;
import java.io.Serializable;

/*	class invariant
		A SimpleList object...
			* is a container of zero or more other objects
			* has no restriction on the type of the objects or 
				number of duplicate objects that it stores. 
			* maintains an iterator that is positioned between 
				either in front of the first item, behind the 
				last item or between two consecutive list items.  
				(The iterator can be thought of as an integer that 
				counts the number of items preceding its position.) */

public class SimpleList implements  Serializable   {
	private ListCell anchor;;
	private ListCell prevCell;
	private int privateCount;

	/*	postcondition
			this is empty (i.e. count() == 0)
			AND iterator == 0  (i.e. the iterator has zero items before it)	*/
	public SimpleList()  {
		anchor = new ListCell(null);
		prevCell = anchor;
		privateCount = 0;
	}  

	/*	postcondition
			iterator == 0  (i.e. positioned before all items in the list) */
	public void start() {
		prevCell = anchor;
	} 

	/*	postcondition
			not isOff  IMPLIES   iterator == old iterator + 1
			AND isOff IMPLIES nothing changes  */
	public void forth()  {
		if (!isOff())
			prevCell = prevCell.next;
	}  

	/*	postcondition
			not isOff  IMPLIES
				result == the item immediately following the iterator ;
				isOff IMPLIES result == null  */
	public Object item()   {
		if (isOff())
			return null;
		else
			return prevCell.next.content;
	} 

	/*	postcondition
			result == ( iterator == count() )   (i.e. result is true exactly when iterator is 
												positioned at the rear of this list)  */
	public boolean isOff()  {
		return (prevCell.next == null);
	} 

	/*	postcondition
			this == the old this list with x inserted at the iterator position 
			AND  count() = old count() + 1
			AND  iterator == old iterator + 1   */
	public void insert(Object f)  {
		ListCell newCell = new ListCell(f);
		newCell.next = prevCell.next;
		prevCell.next = newCell;
		forth();
		privateCount++;
	} 

	/*	postcondition
			not isOff  IMPLIES
				this == old this with the item following old iterator removed  
				AND  count() == old count() - 1 
			isOff IMPLIES nothing changes  */
	public void remove()  {
		if (!isOff())  {
			prevCell.next = prevCell.next.next;
			privateCount--;
		}
	} 

	/*	postcondition
			result == the number of items in this list */
	public int count()  {
		return privateCount;
	}

}